import 'package:flutter/material.dart';

//
// @author vay
// @email hxhvay@126.com
// @date 2020-06-16
//

/// 分隔线组件
class LineWidget extends StatelessWidget {
  /// 组件宽度
  final double width;

  /// 组件高度
  final double height;

  /// 线宽
  final double lineWidth;

  /// 方向
  final Axis direction;

  /// 线的颜色
  final Color color;

  /// 外边距
  final EdgeInsetsGeometry margin;

  /// 虚线
  final bool hasDotted;

  /// 点宽度
  final double dotWidth;

  /// 点间隔
  final double dotSpacing;

  const LineWidget({
    Key key,
    this.width,
    this.height,
    this.lineWidth = 1,
    this.direction = Axis.horizontal,
    this.color = const Color(0xFFDADADA),
    this.margin = EdgeInsets.zero,
    this.hasDotted = false,
    this.dotWidth = 4,
    this.dotSpacing = 4,
  })  : assert(
            (direction == Axis.vertical && height != null) ||
                (direction == Axis.horizontal),
            "direction is vertical, height cannot null."),
        super(key: key);

  /// 是否橫线
  bool get _hasHorizontal => this.direction == Axis.horizontal;

  @override
  Widget build(BuildContext context) {
    double spareWidth = this._hasHorizontal ? double.infinity : this.lineWidth;
    double spareHeight = this._hasHorizontal ? this.lineWidth : double.infinity;
    spareWidth = this.width ?? spareWidth;
    spareHeight = this.height ?? spareHeight;
    return this.hasDotted
        ? _buildDottedLine(spareWidth, spareHeight)
        : _buildLine(spareWidth, spareHeight);
  }

  /// 实线
  Container _buildLine(double width, double height) {
    return Container(
      width: width,
      height: height,
      color: this.color,
      margin: this.margin,
    );
  }

  /// 虚线
  Widget _buildDottedLine(double width, double height) {
    return CustomPaint(
      size: Size(width, height),
      painter: _DottedLinePainter(
        this.direction,
        this.dotWidth,
        this.dotSpacing,
        this.color,
      ),
    );
  }
}

/// 线的绘制
class _DottedLinePainter extends CustomPainter {
  /// 方向
  final Axis direction;

  /// 点宽度
  final double dotWidth;

  /// 点间隔
  final double dotSpacing;

  /// 线的颜色
  final Color color;

  _DottedLinePainter(
    this.direction,
    this.dotWidth,
    this.dotSpacing,
    this.color,
  );

  /// 是否橫线
  bool get _hasHorizontal => this.direction == Axis.horizontal;

  @override
  void paint(Canvas canvas, Size size) {
    var paint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.fill
      ..color = this.color;
    int dotNum = _getDotNum(size);
    for (int i = 0; i < dotNum; i++) {
      double dotStart = i * (this.dotWidth + this.dotSpacing);
      double dotEnd = dotStart + this.dotWidth;
      if (_hasHorizontal) {
        canvas.drawRect(Rect.fromLTRB(dotStart, 0, dotEnd, size.height), paint);
      } else {
        canvas.drawRect(Rect.fromLTRB(0, dotStart, size.width, dotEnd), paint);
      }
    }
  }

  int _getDotNum(Size size) {
    double lineLength = this._hasHorizontal ? size.width : size.height;
    double result = lineLength / _nodeSize;
    return result.toInt();
  }

  double get _nodeSize => this.dotWidth + this.dotSpacing;

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return oldDelegate != this;
  }
}
