import 'package:flutter/material.dart';
import 'package:vaycore/src/utils/widget_utils.dart';
import 'package:vaycore/widget/refresh/custom_refresh_loading.dart';
import 'package:vaycore/widget/refresh/refresh_loading.dart';

//
// @author vay
// @email hxhvay@126.com
// @date 2020-06-21
//

/// 下拉刷新、滚动加载容器
class RefreshContainer extends StatefulWidget {
  final bool enablePullDown;
  final bool enablePullUp;
  final List<Widget> children;
  final CustomRefreshLoading loading;
  final RefreshCallback onRefresh;
  final LoadMoreCallback onLoading;
  final ScrollController controller;
  final double displacement;
  final Color color;
  final Color backgroundColor;
  final double strokeWidth;
  final bool reverse;
  final EdgeInsetsGeometry padding;
  final MainAxisAlignment mainAxisAlignment;
  final MainAxisSize mainAxisSize;
  final CrossAxisAlignment crossAxisAlignment;

  const RefreshContainer({
    Key key,
    this.enablePullDown = true,
    this.enablePullUp = true,
    this.children = const <Widget>[],
    this.loading = const RefreshLoadMore(),
    this.onRefresh,
    this.onLoading,
    this.controller,
    this.displacement = 40.0,
    this.color,
    this.backgroundColor,
    this.strokeWidth = 2.0,
    this.reverse = false,
    this.padding,
    this.mainAxisAlignment = MainAxisAlignment.start,
    this.mainAxisSize = MainAxisSize.max,
    this.crossAxisAlignment = CrossAxisAlignment.start,
  })  : assert(children != null && children.length > 0),
        super(key: key);

  @override
  _RefreshContainerState createState() => _RefreshContainerState();
}

class _RefreshContainerState extends State<RefreshContainer> {
  RefreshStatus refreshStatus;
  LoadStatus loadStatus;

  @override
  void initState() {
    refreshStatus = RefreshStatus.idle;
    loadStatus = LoadStatus.idle;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return widget.enablePullDown ? _buildRefresh() : _buildScrollView();
  }

  Widget _buildRefresh() {
    return RefreshIndicator(
      onRefresh: _handlerRefresh,
      displacement: widget.displacement,
      color: widget.color,
      backgroundColor: widget.backgroundColor,
      strokeWidth: widget.strokeWidth,
      child: _buildScrollView(),
    );
  }

  Widget _buildScrollView() {
    return NotificationListener<ScrollNotification>(
      onNotification: (scroll) {
        if (scroll.depth == 0) {
          _listenScrollToBottom(scroll);
        }
        return false;
      },
      child: SingleChildScrollView(
        child: Column(
          children: <Widget>[
            ...widget.children,
            if (widget.enablePullUp) _buildLoading(),
          ],
          mainAxisAlignment: widget.mainAxisAlignment,
          mainAxisSize: widget.mainAxisSize,
          crossAxisAlignment: widget.crossAxisAlignment,
        ),
        physics: ClampingScrollPhysics(),
        reverse: widget.reverse,
        padding: widget.padding,
        controller: widget.controller,
      ),
    );
  }

  Widget _buildLoading() {
    return WidgetUtils.notNull(
      widget.loading?.builder(loadStatus),
    );
  }

  Future<void> _handlerRefresh() async {
    if (refreshStatus == RefreshStatus.refreshing ||
        loadStatus == LoadStatus.loading) {
      return Future.value();
    }
    refreshStatus = RefreshStatus.refreshing;
    if (widget.onRefresh != null) {
      await widget.onRefresh();
    }
    refreshStatus = RefreshStatus.idle;
    loadStatus = LoadStatus.idle;
    setState(() {});
    return Future.value();
  }

  void _handlerLoadMore() async {
    if (refreshStatus == RefreshStatus.refreshing ||
        loadStatus != LoadStatus.idle) {
      return;
    }

    loadStatus = LoadStatus.loading;
    if (widget.onLoading != null) {
      bool moreData = await widget.onLoading();
      loadStatus = moreData ? LoadStatus.idle : LoadStatus.noMore;
    }
    setState(() {});
  }

  void _listenScrollToBottom(ScrollNotification scroll) {
    ScrollMetrics metrics = scroll.metrics;
    if (metrics.pixels == metrics.maxScrollExtent) {
      _handlerLoadMore();
    }
  }
}

/// header state
enum RefreshStatus {
  idle,
  refreshing,
}

///  footer state
enum LoadStatus {
  idle,
  loading,
  noMore,
}

/// 加载更多回调（true==有更多数据，false==没有更多数据）
typedef LoadMoreCallback = Future<bool> Function();
